@@ -53,8 +53,8 @@ logger = logging.getLogger("wgpu")  # noqa
 apidiff = ApiDiff()
 
 # The wgpu-native version that we target/expect
-__version__ = "0.8.0.2"
-__commit_sha__ = "66a4139579f2499a67b7ade1a6c3bcb414046c64"
+__version__ = "0.9.2.2"
+__commit_sha__ = "b10496e7eed9349f0fd541e6dfe5029cb436de74"
 version_info = tuple(map(int, __version__.split(".")))
 check_expected_version(version_info)  # produces a warning on mismatch
 
@@ -207,22 +207,37 @@ class GPU(base.GPU):
         # Get surface id that the adapter must be compatible with. If we
         # don't pass a valid surface id, there is no guarantee we'll be
         # able to create a swapchain for it (from this adapter).
-        if canvas is None:
-            surface_id = ffi.NULL
-        else:
-            surface_id = get_surface_id_from_canvas(canvas)
+        surface_id = ffi.NULL
+        if canvas is not None:
+            window_id = canvas.get_window_id()
+            if window_id is not None:  # e.g. could be an off-screen canvas
+                surface_id = get_surface_id_from_canvas(canvas)
+                surface_id = ffi.NULL  # off-screen canvas
 
-        # Force Vulkan on Windows, to avoid DX12 which seems to ignore
-        # the NVidia control panel settings. I guess Vulkan is more
-        # mature than Metal too, so let's just force that for now.
+        # Try to read the WGPU_BACKEND_TYPE environment variable to see
+        # if a backend should be forced. When you run into trouble with
+        # the automatic selection of wgpu, you can use this variable
+        # to force a specific backend. For instance, on Windows you
+        # might want to force Vulkan, to avoid DX12 which seems to ignore
+        # the NVidia control panel settings.
         # See https://github.com/gfx-rs/wgpu/issues/1416
-        # force_backend = lib.WGPUBackendType_Vulkan
-        force_backend = enum_str2int["BackendType"]["Vulkan"]
+        force_backend = os.getenv("WGPU_BACKEND_TYPE", "")
+        backend = enum_str2int["BackendType"]["Null"]
+        if force_backend:
+            try:
+                backend = enum_str2int["BackendType"][force_backend]
+            except KeyError:
+                logger.warn(
+                    f"Invalid value for WGPU_BACKEND_TYPE: '{force_backend}'.\n"
+                    f"Valid values are: {list(enum_str2int['BackendType'].keys())}"
+                )
+            else:
+                logger.warn(f"Forcing backend: {force_backend} ({backend})")
 
         # H: chain: WGPUChainedStruct, backend: WGPUBackendType
         extras = new_struct_p(
             "WGPUAdapterExtras *",
-            backend=force_backend,
+            backend=backend,
             # not used: chain
         )
         extras.chain.sType = lib.WGPUSType_AdapterExtras
@@ -305,7 +320,57 @@ class GPU(base.GPU):
 
 
 class GPUCanvasContext(base.GPUCanvasContext):
-    pass
+    def __init__(self, canvas):
+        super().__init__(canvas)
+        self._surface_size = (-1, -1)
+        self._surface_id = None
+        self._internal = None
+
+    def get_current_texture(self):
+        if self._device is None:
+            raise RuntimeError(
+                "Preset context must be configured before get_current_texture()."
+            )
+        self._create_native_swap_chain_if_needed()
+        # H: WGPUTextureView f(WGPUSwapChain swapChain)
+        view_id = lib.wgpuSwapChainGetCurrentTextureView(self._internal)
+        size = self._surface_size[0], self._surface_size[1], 1
+        return GPUTextureView("swap_chain", view_id, self._device, None, size)
+
+    def present(self):
+        if self._internal is not None and lib is not None:
+            # H: void f(WGPUSwapChain swapChain)
+            lib.wgpuSwapChainPresent(self._internal)
+
+    def _create_native_swap_chain_if_needed(self):
+        canvas = self._get_canvas()
+        psize = canvas.get_physical_size()
+        if psize == self._surface_size:
+            return
+        self._surface_size = psize
+
+        # logger.info(str((psize, canvas.get_logical_size(), canvas.get_pixel_ratio())))
+
+        # H: nextInChain: WGPUChainedStruct *, label: char *, usage: WGPUTextureUsageFlags/int, format: WGPUTextureFormat, width: int, height: int, presentMode: WGPUPresentMode
+        struct = new_struct_p(
+            "WGPUSwapChainDescriptor *",
+            usage=self._usage,
+            format=self._format,
+            width=max(1, psize[0]),
+            height=max(1, psize[1]),
+            presentMode=2,
+            # not used: nextInChain
+            # not used: label
+        )
+        # present_mode -> 0: Immediate, 1: Mailbox, 2: Fifo
+
+        if self._surface_id is None:
+            self._surface_id = get_surface_id_from_canvas(canvas)
+
+        # H: WGPUSwapChain f(WGPUDevice device, WGPUSurface surface, WGPUSwapChainDescriptor const * descriptor)
+        self._internal = lib.wgpuDeviceCreateSwapChain(
+            self._device._internal, self._surface_id, struct
+        )
 
 
 class GPUObjectBase(base.GPUObjectBase):
@@ -317,12 +382,10 @@ class GPUAdapter(base.GPUAdapter):
         self,
         *,
         label="",
-        non_guaranteed_features: "List[enums.FeatureName]" = [],
-        non_guaranteed_limits: "Dict[str, int]" = {},
+        required_features: "List[enums.FeatureName]" = [],
+        required_limits: "Dict[str, int]" = {},
     ):
-        return self._request_device(
-            label, non_guaranteed_features, non_guaranteed_limits, ""
-        )
+        return self._request_device(label, required_features, required_limits, "")
 
     @apidiff.add("a sweet bonus feature from wgpu-native")
     def request_device_tracing(
@@ -330,8 +393,8 @@ class GPUAdapter(base.GPUAdapter):
         trace_path,
         *,
         label="",
-        non_guaranteed_features: "list(enums.FeatureName)" = [],
-        non_guaranteed_limits: "Dict[str, int]" = {},
+        required_features: "list(enums.FeatureName)" = [],
+        required_limits: "Dict[str, int]" = {},
     ):
         """Write a trace of all commands to a file so it can be reproduced
         elsewhere. The trace is cross-platform!
@@ -341,7 +404,7 @@ class GPUAdapter(base.GPUAdapter):
         elif os.listdir(trace_path):
             logger.warning(f"Trace directory not empty: {trace_path}")
         return self._request_device(
-            label, non_guaranteed_features, non_guaranteed_limits, trace_path
+            label, required_features, required_limits, trace_path
         )
 
     def _request_device(self, label, features, limits, trace_path):
@@ -415,15 +478,15 @@ class GPUAdapter(base.GPUAdapter):
         self,
         *,
         label="",
-        non_guaranteed_features: "List[enums.FeatureName]" = [],
-        non_guaranteed_limits: "Dict[str, int]" = {},
+        required_features: "List[enums.FeatureName]" = [],
+        required_limits: "Dict[str, int]" = {},
     ):
         return self._request_device(
-            label, non_guaranteed_features, non_guaranteed_limits, ""
+            label, required_features, required_limits, ""
         )  # no-cover
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_adapter_destroy(internal)
 
@@ -495,12 +558,12 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         size = _tuple_from_tuple_or_dict(
             size, ("width", "height", "depth_or_array_layers")
         )
-        # H: width: int, height: int, depth: int
+        # H: width: int, height: int, depthOrArrayLayers: int
         c_size = new_struct(
             "WGPUExtent3D",
             width=size[0],
             height=size[1],
-            depth=size[2],
+            depthOrArrayLayers=size[2],
         )
         # H: nextInChain: WGPUChainedStruct *, label: char *, usage: WGPUTextureUsageFlags/int, dimension: WGPUTextureDimension, size: WGPUExtent3D, format: WGPUTextureFormat, mipLevelCount: int, sampleCount: int
         struct = new_struct_p(
@@ -538,7 +601,7 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         min_filter: "enums.FilterMode" = "nearest",
         mipmap_filter: "enums.FilterMode" = "nearest",
         lod_min_clamp: float = 0,
-        lod_max_clamp: float = 0xFFFFFFFF,
+        lod_max_clamp: float = 32,
         compare: "enums.CompareFunction" = None,
         max_anisotropy: int = 1,
     ):
@@ -1060,6 +1123,8 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         color_formats: "List[enums.TextureFormat]",
         depth_stencil_format: "enums.TextureFormat" = None,
         sample_count: int = 1,
+        depth_read_only: bool = False,
+        stencil_read_only: bool = False,
     ):
         raise NotImplementedError()
 
@@ -1075,7 +1140,7 @@ class GPUDevice(base.GPUDevice, GPUObjectBase):
         raise NotImplementedError()
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_device_destroy(internal)
 
@@ -1164,7 +1229,7 @@ class GPUBuffer(base.GPUBuffer, GPUObjectBase):
         self._destroy()  # no-cover
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             # H: void f(WGPUBuffer buffer)
             lib.wgpuBufferDestroy(internal)
@@ -1221,7 +1286,7 @@ class GPUTexture(base.GPUTexture, GPUObjectBase):
         self._destroy()  # no-cover
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             # H: void f(WGPUTexture texture)
             lib.wgpuTextureDestroy(internal)
@@ -1229,35 +1294,35 @@ class GPUTexture(base.GPUTexture, GPUObjectBase):
 
 class GPUTextureView(base.GPUTextureView, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_texture_view_destroy(internal, False)
 
 
 class GPUSampler(base.GPUSampler, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_sampler_destroy(internal)
 
 
 class GPUBindGroupLayout(base.GPUBindGroupLayout, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_bind_group_layout_destroy(internal)
 
 
 class GPUBindGroup(base.GPUBindGroup, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_bind_group_destroy(internal)
 
 
 class GPUPipelineLayout(base.GPUPipelineLayout, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_pipeline_layout_destroy(internal)
 
@@ -1267,7 +1332,7 @@ class GPUShaderModule(base.GPUShaderModule, GPUObjectBase):
         return super().compilation_info()
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_shader_module_destroy(internal)
 
@@ -1278,21 +1343,21 @@ class GPUPipelineBase(base.GPUPipelineBase):
 
 class GPUComputePipeline(base.GPUComputePipeline, GPUPipelineBase, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_compute_pipeline_destroy(internal)
 
 
 class GPURenderPipeline(base.GPURenderPipeline, GPUPipelineBase, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_render_pipeline_destroy(internal)
 
 
 class GPUCommandBuffer(base.GPUCommandBuffer, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_command_buffer_destroy(internal)
 
@@ -1345,10 +1410,10 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
                 b=clr[2],
                 a=clr[3],
             )
-            # H: attachment: WGPUTextureView, resolveTarget: WGPUTextureView, loadOp: WGPULoadOp, storeOp: WGPUStoreOp, clearColor: WGPUColor
+            # H: view: WGPUTextureView, resolveTarget: WGPUTextureView, loadOp: WGPULoadOp, storeOp: WGPUStoreOp, clearColor: WGPUColor
             c_attachment = new_struct(
-                "WGPURenderPassColorAttachmentDescriptor",
-                attachment=texture_view_id,
+                "WGPURenderPassColorAttachment",
+                view=texture_view_id,
                 resolveTarget=c_resolve_target,
                 loadOp=c_load_op,
                 storeOp=color_attachment.get("store_op", "store"),
@@ -1357,7 +1422,7 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
             )
             c_color_attachments_list.append(c_attachment)
         c_color_attachments_array = ffi.new(
-            "WGPURenderPassColorAttachmentDescriptor []", c_color_attachments_list
+            "WGPURenderPassColorAttachment []", c_color_attachments_list
         )
 
         c_depth_stencil_attachment = ffi.NULL
@@ -1369,10 +1434,10 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
             c_stencil_load_op, c_stencil_clear = _loadop_and_clear_from_value(
                 depth_stencil_attachment["stencil_load_value"]
             )
-            # H: attachment: WGPUTextureView, depthLoadOp: WGPULoadOp, depthStoreOp: WGPUStoreOp, clearDepth: float, depthReadOnly: bool, stencilLoadOp: WGPULoadOp, stencilStoreOp: WGPUStoreOp, clearStencil: int, stencilReadOnly: bool
+            # H: view: WGPUTextureView, depthLoadOp: WGPULoadOp, depthStoreOp: WGPUStoreOp, clearDepth: float, depthReadOnly: bool, stencilLoadOp: WGPULoadOp, stencilStoreOp: WGPUStoreOp, clearStencil: int, stencilReadOnly: bool
             c_depth_stencil_attachment = new_struct_p(
-                "WGPURenderPassDepthStencilAttachmentDescriptor *",
-                attachment=depth_stencil_attachment["view"]._internal,
+                "WGPURenderPassDepthStencilAttachment *",
+                view=depth_stencil_attachment["view"]._internal,
                 depthLoadOp=c_depth_load_op,
                 depthStoreOp=depth_stencil_attachment["depth_store_op"],
                 clearDepth=float(c_depth_clear),
@@ -1385,7 +1450,7 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
                 ),
             )
 
-        # H: nextInChain: WGPUChainedStruct *, label: char *, colorAttachmentCount: int, colorAttachments: WGPURenderPassColorAttachmentDescriptor *, depthStencilAttachment: WGPURenderPassDepthStencilAttachmentDescriptor *, occlusionQuerySet: WGPUQuerySet
+        # H: nextInChain: WGPUChainedStruct *, label: char *, colorAttachmentCount: int, colorAttachments: WGPURenderPassColorAttachment *, depthStencilAttachment: WGPURenderPassDepthStencilAttachment *, occlusionQuerySet: WGPUQuerySet
         struct = new_struct_p(
             "WGPURenderPassDescriptor *",
             label=to_c_label(label),
@@ -1463,12 +1528,12 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         size = _tuple_from_tuple_or_dict(
             copy_size, ("width", "height", "depth_or_array_layers")
         )
-        # H: width: int, height: int, depth: int
+        # H: width: int, height: int, depthOrArrayLayers: int
         c_copy_size = new_struct_p(
             "WGPUExtent3D *",
             width=size[0],
             height=size[1],
-            depth=size[2],
+            depthOrArrayLayers=size[2],
         )
 
         # H: void f(WGPUCommandEncoder commandEncoder, WGPUImageCopyBuffer const * source, WGPUImageCopyTexture const * destination, WGPUExtent3D const * copySize)
@@ -1523,12 +1588,12 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         size = _tuple_from_tuple_or_dict(
             copy_size, ("width", "height", "depth_or_array_layers")
         )
-        # H: width: int, height: int, depth: int
+        # H: width: int, height: int, depthOrArrayLayers: int
         c_copy_size = new_struct_p(
             "WGPUExtent3D *",
             width=size[0],
             height=size[1],
-            depth=size[2],
+            depthOrArrayLayers=size[2],
         )
 
         # H: void f(WGPUCommandEncoder commandEncoder, WGPUImageCopyTexture const * source, WGPUImageCopyBuffer const * destination, WGPUExtent3D const * copySize)
@@ -1585,12 +1650,12 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         size = _tuple_from_tuple_or_dict(
             copy_size, ("width", "height", "depth_or_array_layers")
         )
-        # H: width: int, height: int, depth: int
+        # H: width: int, height: int, depthOrArrayLayers: int
         c_copy_size = new_struct_p(
             "WGPUExtent3D *",
             width=size[0],
             height=size[1],
-            depth=size[2],
+            depthOrArrayLayers=size[2],
         )
 
         # H: void f(WGPUCommandEncoder commandEncoder, WGPUImageCopyTexture const * source, WGPUImageCopyTexture const * destination, WGPUExtent3D const * copySize)
@@ -1635,7 +1700,7 @@ class GPUCommandEncoder(base.GPUCommandEncoder, GPUObjectBase):
         raise NotImplementedError()
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_command_encoder_destroy(internal)
 
@@ -1740,7 +1805,7 @@ class GPUComputePassEncoder(
         raise NotImplementedError()
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_compute_pass_destroy(internal)
 
@@ -1753,7 +1818,7 @@ class GPURenderEncoderBase(base.GPURenderEncoderBase):
         # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPURenderPipeline pipeline)
         lib.wgpuRenderPassEncoderSetPipeline(self._internal, pipeline_id)
 
-    def set_index_buffer(self, buffer, index_format, offset=0, size=0):
+    def set_index_buffer(self, buffer, index_format, offset=0, size=None):
         if not size:
             size = buffer.size - offset
         c_index_format = enummap[f"IndexFormat.{index_format}"]
@@ -1762,7 +1827,7 @@ class GPURenderEncoderBase(base.GPURenderEncoderBase):
             self._internal, buffer._internal, c_index_format, int(offset), int(size)
         )
 
-    def set_vertex_buffer(self, slot, buffer, offset=0, size=0):
+    def set_vertex_buffer(self, slot, buffer, offset=0, size=None):
         if not size:
             size = buffer.size - offset
         # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t slot, WGPUBuffer buffer, uint64_t offset, uint64_t size)
@@ -1809,7 +1874,7 @@ class GPURenderEncoderBase(base.GPURenderEncoderBase):
         )
 
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_render_pass_destroy(internal)
 
@@ -1849,7 +1914,7 @@ class GPURenderPassEncoder(
             a=color[3],
         )
         # H: void f(WGPURenderPassEncoder renderPassEncoder, WGPUColor const * color)
-        lib.wgpuRenderPassEncoderSetBlendColor(self._internal, c_color)
+        lib.wgpuRenderPassEncoderSetBlendConstant(self._internal, c_color)
 
     def set_stencil_reference(self, reference):
         # H: void f(WGPURenderPassEncoder renderPassEncoder, uint32_t reference)
@@ -2000,12 +2065,12 @@ class GPUQueue(base.GPUQueue, GPUObjectBase):
         size = _tuple_from_tuple_or_dict(
             size, ("width", "height", "depth_or_array_layers")
         )
-        # H: width: int, height: int, depth: int
+        # H: width: int, height: int, depthOrArrayLayers: int
         c_size = new_struct_p(
             "WGPUExtent3D *",
             width=size[0],
             height=size[1],
-            depth=size[2],
+            depthOrArrayLayers=size[2],
         )
 
         # H: void f(WGPUQueue queue, WGPUImageCopyTexture const * destination, void const * data, size_t dataSize, WGPUTextureDataLayout const * dataLayout, WGPUExtent3D const * writeSize)
@@ -2075,66 +2140,9 @@ class GPUQueue(base.GPUQueue, GPUObjectBase):
         raise NotImplementedError()
 
 
-class GPUSwapChain(base.GPUSwapChain, GPUObjectBase):
-    def __init__(
-        self, label, internal, device, canvas, format, usage, compositing_alpha_mode
-    ):
-        super().__init__(
-            label, internal, device, canvas, format, usage, compositing_alpha_mode
-        )
-        assert internal is None  # we set it later
-        self._surface_size = (-1, -1)
-        self._surface_id = None
-        self._create_native_swap_chain_if_needed()
-
-    def _create_native_swap_chain_if_needed(self):
-        canvas = self._canvas
-        psize = canvas.get_physical_size()
-        if psize == self._surface_size:
-            return
-        self._surface_size = psize
-
-        # logger.info(str((psize, canvas.get_logical_size(), canvas.get_pixel_ratio())))
-
-        # H: nextInChain: WGPUChainedStruct *, label: char *, usage: WGPUTextureUsageFlags/int, format: WGPUTextureFormat, width: int, height: int, presentMode: WGPUPresentMode
-        struct = new_struct_p(
-            "WGPUSwapChainDescriptor *",
-            usage=self._usage,
-            format=self._format,
-            width=max(1, psize[0]),
-            height=max(1, psize[1]),
-            presentMode=2,
-            # not used: nextInChain
-            # not used: label
-        )
-        # present_mode -> 0: Immediate, 1: Mailbox, 2: Fifo
-
-        if self._surface_id is None:
-            self._surface_id = get_surface_id_from_canvas(canvas)
-
-        # H: WGPUSwapChain f(WGPUDevice device, WGPUSurface surface, WGPUSwapChainDescriptor const * descriptor)
-        self._internal = lib.wgpuDeviceCreateSwapChain(
-            self._device._internal, self._surface_id, struct
-        )
-
-    def __enter__(self):
-        # Get the current texture view, and make sure it is presented when done
-        self._create_native_swap_chain_if_needed()
-        # H: WGPUTextureView f(WGPUSwapChain swapChain)
-        view_id = lib.wgpuSwapChainGetCurrentTextureView(self._internal)
-        size = self._surface_size[0], self._surface_size[1], 1
-        return GPUTextureView("swap_chain", view_id, self._device, None, size)
-
-    def __exit__(self, type, value, tb):
-        # Present the current texture
-        # H: void f(WGPUSwapChain swapChain)
-        lib.wgpuSwapChainPresent(self._internal)
-        # todo: we used to get info on the status here
-
-
 class GPURenderBundle(base.GPURenderBundle, GPUObjectBase):
     def _destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             internal  # doesnotexist.wgpu_render_bundle_destroy(internal)
 
@@ -2163,7 +2171,7 @@ class GPUQuerySet(base.GPUQuerySet, GPUObjectBase):
     pass
 
     def destroy(self):
-        if self._internal is not None:
+        if self._internal is not None and lib is not None:
             self._internal, internal = None, self._internal
             # H: void f(WGPUQuerySet querySet)
             lib.wgpuQuerySetDestroy(internal)
@@ -2181,8 +2189,9 @@ class GPUExternalTexture(base.GPUExternalTexture, GPUObjectBase):
 
 
 def _copy_docstrings():
+    base_classes = GPUObjectBase, GPUCanvasContext
     for ob in globals().values():
-        if not (isinstance(ob, type) and issubclass(ob, GPUObjectBase)):
+        if not (isinstance(ob, type) and issubclass(ob, base_classes)):
             continue
         elif ob.__module__ != __name__:
             continue  # no-cover
